@@ -277,7 +277,7 @@ int GPTData::Verify(void) {
    // Check that partitions are aligned on proper boundaries (for WD Advanced
    // Format and similar disks)....
    for (i = 0; i < numParts; i++) {
-      if ((partitions[i].GetFirstLBA() % sectorAlignment) != 0) {
+      if ((partitions[i].IsUsed()) && (partitions[i].GetFirstLBA() % sectorAlignment) != 0) {
          cout << "\nCaution: Partition " << i + 1 << " doesn't begin on a "
               << sectorAlignment << "-sector boundary. This may\nresult "
               << "in degraded performance on some modern (2009 and later) hard disks.\n";
@@ -316,11 +316,13 @@ int GPTData::CheckGPTSize(void) {
    firstUsedBlock = UINT64_MAX;
    lastUsedBlock = 0;
    for (i = 0; i < numParts; i++) {
-      if ((partitions[i].GetFirstLBA() < firstUsedBlock) &&
-           (partitions[i].GetFirstLBA() != 0))
-         firstUsedBlock = partitions[i].GetFirstLBA();
-      if (partitions[i].GetLastLBA() > lastUsedBlock)
-         lastUsedBlock = partitions[i].GetLastLBA();
+      if (partitions[i].IsUsed()) {
+         if (partitions[i].GetFirstLBA() < firstUsedBlock)
+            firstUsedBlock = partitions[i].GetFirstLBA();
+         if (partitions[i].GetLastLBA() > lastUsedBlock) {
+            lastUsedBlock = partitions[i].GetLastLBA();
+         } // if
+      } // if
    } // for
 
    // If the disk size is 0 (the default), then it means that various
@@ -516,7 +518,7 @@ void GPTData::RebuildMainHeader(void) {
    mainHeader.partitionEntriesCRC = secondHeader.partitionEntriesCRC;
    memcpy(mainHeader.reserved2, secondHeader.reserved2, sizeof(mainHeader.reserved2));
    mainCrcOk = secondCrcOk;
-   SetGPTSize(mainHeader.numParts);
+   SetGPTSize(mainHeader.numParts, 0);
 } // GPTData::RebuildMainHeader()
 
 // Rebuild the secondary GPT header, using the main header as a model.
@@ -537,7 +539,7 @@ void GPTData::RebuildSecondHeader(void) {
    secondHeader.partitionEntriesCRC = mainHeader.partitionEntriesCRC;
    memcpy(secondHeader.reserved2, mainHeader.reserved2, sizeof(secondHeader.reserved2));
    secondCrcOk = mainCrcOk;
-   SetGPTSize(secondHeader.numParts);
+   SetGPTSize(secondHeader.numParts, 0);
 } // GPTData::RebuildSecondHeader()
 
 // Search for hybrid MBR entries that have no corresponding GPT partition.
@@ -555,7 +557,7 @@ int GPTData::FindHybridMismatches(void) {
          mbrLast = mbrFirst + (uint64_t) protectiveMBR.GetLength(i) - UINT64_C(1);
          do {
             if ((partitions[j].GetFirstLBA() == mbrFirst) &&
-                (partitions[j].GetLastLBA() == mbrLast))
+                (partitions[j].GetLastLBA() == mbrLast) && (partitions[j].IsUsed()))
                found = 1;
             j++;
          } while ((!found) && (j < numParts));
@@ -585,7 +587,8 @@ int GPTData::FindOverlaps(void) {
 
    for (i = 1; i < numParts; i++) {
       for (j = 0; j < i; j++) {
-         if (partitions[i].DoTheyOverlap(partitions[j])) {
+         if ((partitions[i].IsUsed()) && (partitions[j].IsUsed()) &&
+             (partitions[i].DoTheyOverlap(partitions[j]))) {
             problems++;
             cout << "\nProblem: partitions " << i + 1 << " and " << j + 1 << " overlap:\n";
             cout << "  Partition " << i + 1 << ": " << partitions[i].GetFirstLBA()
@@ -608,13 +611,15 @@ int GPTData::FindInsanePartitions(void) {
    int problems = 0;
 
    for (i = 0; i < numParts; i++) {
-      if (partitions[i].GetFirstLBA() > partitions[i].GetLastLBA()) {
-         problems++;
-         cout << "\nProblem: partition " << i + 1 << " ends before it begins.\n";
-      } // if
-      if (partitions[i].GetLastLBA() >= diskSize) {
-         problems++;
-      cout << "\nProblem: partition " << i + 1 << " is too big for the disk.\n";
+      if (partitions[i].IsUsed()) {
+         if (partitions[i].GetFirstLBA() > partitions[i].GetLastLBA()) {
+            problems++;
+            cout << "\nProblem: partition " << i + 1 << " ends before it begins.\n";
+         } // if
+         if (partitions[i].GetLastLBA() >= diskSize) {
+            problems++;
+         cout << "\nProblem: partition " << i + 1 << " is too big for the disk.\n";
+         } // if
       } // if
    } // for
    return problems;
@@ -877,7 +882,7 @@ int GPTData::LoadHeader(struct GPTHeader *header, DiskIO & disk, uint64_t sector
    *crcOk = CheckHeaderCRC(&tempHeader);
 
    if (allOK && (numParts != tempHeader.numParts) && *crcOk) {
-      allOK = SetGPTSize(tempHeader.numParts);
+      allOK = SetGPTSize(tempHeader.numParts, 0);
    }
 
    *header = tempHeader;
@@ -900,7 +905,7 @@ int GPTData::LoadPartitionTable(const struct GPTHeader & header, DiskIO & disk,
          retval = disk.Seek(sector);
       } // if/else
       if (retval == 1)
-         retval = SetGPTSize(header.numParts);
+         retval = SetGPTSize(header.numParts, 0);
       if (retval == 1) {
          sizeOfParts = header.numParts * header.sizeOfPartitionEntries;
          if (disk.Read(partitions, sizeOfParts) != (int) sizeOfParts) {
@@ -965,14 +970,16 @@ int GPTData::CheckTable(struct GPTHeader *header) {
    return allOK;
 } // GPTData::CheckTable()
 
-// Writes GPT (and protective MBR) to disk. If quiet==1, 
+// Writes GPT (and protective MBR) to disk. If quiet==1, moves the second
+// header later on the disk without asking for permission, if necessary, and
+// doesn't confirm the operation before writing. If quiet==0, asks permission
+// before moving the second header and asks for final confirmation of any
+// write.
 // Returns 1 on successful write, 0 if there was a problem.
 int GPTData::SaveGPTData(int quiet) {
-   int allOK = 1, littleEndian;
+   int allOK = 1, syncIt = 1;
    char answer;
 
-   littleEndian = IsLittleEndian();
-
    // First do some final sanity checks....
 
    // This test should only fail on read-only disks....
@@ -1028,7 +1035,7 @@ int GPTData::SaveGPTData(int quiet) {
            << "PARTITIONS!!\n\nDo you want to proceed? ";
       answer = GetYN();
       if (answer == 'Y') {
-         cout << "OK; writing new GUID partition table (GPT).\n";
+         cout << "OK; writing new GUID partition table (GPT) to " << myDisk.GetName() << ".\n";
       } else {
          allOK = 0;
       } // if/else
@@ -1039,9 +1046,11 @@ int GPTData::SaveGPTData(int quiet) {
       if (myDisk.OpenForWrite()) {
          // As per UEFI specs, write the secondary table and GPT first....
          allOK = SavePartitionTable(myDisk, secondHeader.partitionEntriesLBA);
-         if (!allOK)
+         if (!allOK) {
             cerr << "Unable to save backup partition table! Perhaps the 'e' option on the experts'\n"
                  << "menu will resolve this problem.\n";
+            syncIt = 0;
+         } // if
 
          // Now write the secondary GPT header...
          allOK = allOK && SaveHeader(&secondHeader, myDisk, mainHeader.backupLBA);
@@ -1056,15 +1065,21 @@ int GPTData::SaveGPTData(int quiet) {
          allOK = allOK && protectiveMBR.WriteMBRData(&myDisk);
 
          // re-read the partition table
-         if (allOK) {
+         // Note: Done even if some write operations failed, but not if all of them failed.
+         // Done this way because I've received one problem report from a user one whose
+         // system the MBR write failed but everything else was OK (on a GPT disk under
+         // Windows), and the failure to sync therefore caused Windows to restore the
+         // original partition table from its cache. OTOH, such restoration might be
+         // desirable if the error occurs later; but that seems unlikely unless the initial
+         // write fails....
+         if (syncIt)
             myDisk.DiskSync();
-         } // if
 
          if (allOK) { // writes completed OK
             cout << "The operation has completed successfully.\n";
          } else {
             cerr << "Warning! An error was reported when writing the partition table! This error\n"
-                 << "MIGHT be harmless, but you may have trashed the disk!\n";
+                 << "MIGHT be harmless, or the disk might be damaged! Checking it is advisable.\n";
          } // if/else
 
          myDisk.Close();
@@ -1172,14 +1187,10 @@ int GPTData::SavePartitionTable(DiskIO & disk, uint64_t sector) {
 // set of partitions.
 int GPTData::LoadGPTBackup(const string & filename) {
    int allOK = 1, val, err;
-   uint32_t sizeOfEntries;
-   int littleEndian = 1, shortBackup = 0;
+   int shortBackup = 0;
    DiskIO backupFile;
 
    if (backupFile.OpenForRead(filename)) {
-      if (IsLittleEndian() == 0)
-         littleEndian = 0;
-
       // Let the MBRData class load the saved MBR...
       protectiveMBR.ReadMBRData(&backupFile, 0); // 0 = don't check block size
       protectiveMBR.SetDisk(&myDisk);
@@ -1206,11 +1217,9 @@ int GPTData::LoadGPTBackup(const string & filename) {
       // this check!
       if ((val = CheckHeaderValidity()) > 0) {
          if (val == 2) { // only backup header seems to be good
-            SetGPTSize(secondHeader.numParts);
-            sizeOfEntries = secondHeader.sizeOfPartitionEntries;
+            SetGPTSize(secondHeader.numParts, 0);
          } else { // main header is OK
-            SetGPTSize(mainHeader.numParts);
-            sizeOfEntries = mainHeader.sizeOfPartitionEntries;
+            SetGPTSize(mainHeader.numParts, 0);
          } // if/else
 
          if (secondHeader.currentLBA != diskSize - UINT64_C(1)) {
@@ -1595,19 +1604,21 @@ int GPTData::OnePartToMBR(uint32_t gptPart, int mbrPart) {
  **********************************************************************/
 
 // Resizes GPT to specified number of entries. Creates a new table if
-// necessary, copies data if it already exists. Returns 1 if all goes
-// well, 0 if an error is encountered.
-int GPTData::SetGPTSize(uint32_t numEntries) {
+// necessary, copies data if it already exists. If fillGPTSectors is 1
+// (the default), rounds numEntries to fill all the sectors necessary to
+// hold the GPT.
+// Returns 1 if all goes well, 0 if an error is encountered.
+int GPTData::SetGPTSize(uint32_t numEntries, int fillGPTSectors) {
    GPTPart* newParts;
-   uint32_t i, high, copyNum;
+   uint32_t i, high, copyNum, entriesPerSector;
    int allOK = 1;
 
    // First, adjust numEntries upward, if necessary, to get a number
    // that fills the allocated sectors
-   i = blockSize / GPT_SIZE;
-   if ((numEntries % i) != 0) {
+   entriesPerSector = blockSize / GPT_SIZE;
+   if (fillGPTSectors && ((numEntries % entriesPerSector) != 0)) {
       cout << "Adjusting GPT size from " << numEntries << " to ";
-      numEntries = ((numEntries / i) + 1) * i;
+      numEntries = ((numEntries / entriesPerSector) + 1) * entriesPerSector;
       cout << numEntries << " to fill the sector\n";
    } // if
 
@@ -1643,7 +1654,7 @@ int GPTData::SetGPTSize(uint32_t numEntries) {
             partitions = newParts;
          } // if/else existing partitions
          numParts = numEntries;
-         mainHeader.firstUsableLBA = ((numEntries * GPT_SIZE) / blockSize) + 2 ;
+         mainHeader.firstUsableLBA = ((numEntries * GPT_SIZE) / blockSize) + (((numEntries * GPT_SIZE) % blockSize) != 0) + 2 ;
          secondHeader.firstUsableLBA = mainHeader.firstUsableLBA;
          MoveSecondHeaderToEnd();
          if (diskSize > 0)
@@ -1842,7 +1853,7 @@ int GPTData::SetPartitionGUID(uint32_t pn, GUIDData theGUID) {
    int retval = 0;
 
    if (pn < numParts) {
-      if (partitions[pn].GetFirstLBA() != UINT64_C(0)) {
+      if (partitions[pn].IsUsed()) {
          partitions[pn].SetUniqueGUID(theGUID);
          retval = 1;
       } // if
@@ -1952,7 +1963,7 @@ int GPTData::GetPartRange(uint32_t *low, uint32_t *high) {
    *low = numParts + 1; // code for "not found"
    *high = 0;
    for (i = 0; i < numParts; i++) {
-      if (partitions[i].GetFirstLBA() != UINT64_C(0)) { // it exists
+      if (partitions[i].IsUsed()) { // it exists
          *high = i; // since we're counting up, set the high value
          // Set the low value only if it's not yet found...
          if (*low == (numParts + 1)) *low = i;
@@ -2020,7 +2031,7 @@ uint64_t GPTData::FindFirstAvailable(uint64_t start) {
    do {
       firstMoved = 0;
       for (i = 0; i < numParts; i++) {
-         if ((first >= partitions[i].GetFirstLBA()) &&
+         if ((partitions[i].IsUsed()) && (first >= partitions[i].GetFirstLBA()) &&
              (first <= partitions[i].GetLastLBA())) { // in existing part.
             first = partitions[i].GetLastLBA() + 1;
             firstMoved = 1;
@@ -2306,7 +2317,8 @@ int GPTData::ManageAttributes(int partNum, const string & command, const string
 
 // Show all attributes for a specified partition....
 void GPTData::ShowAttributes(const uint32_t partNum) {
-   partitions[partNum].ShowAttributes(partNum);
+   if (partitions[partNum].IsUsed())
+      partitions[partNum].ShowAttributes(partNum);
 } // GPTData::ShowAttributes
 
 // Show whether a single attribute bit is set (terse output)...
